import click

from infra_buddy.aws.cloudformation import CloudFormationBuddy
from infra_buddy.commandline import cli
from infra_buddy.context.deploy_ctx import DeployContext
from infra_buddy.commands.deploy_cloudformation import command as deploy_cf
from infra_buddy.deploy.cloudformation_deploy import CloudFormationDeploy
from infra_buddy.template.template import NamedLocalTemplate
from infra_buddy.utility import print_utility


@cli.command(name='validate-template')
@click.option("--template-directory",type=click.Path(exists=True), help="The directory containing "
                                                                                "the service template.")
@click.option("--service-type", help="The service-type that corresponds with the provided template directory.")
@click.pass_obj
def deploy_cloudformation(deploy_ctx,service_template_directory,service_type):
    # type: (DeployContext,str,str) -> None
    do_command(deploy_ctx,service_template_directory,service_type)


def do_command(deploy_ctx,service_template_directory=None,service_type=None):
    # type: (DeployContext,[str or None],str) -> None
    if service_template_directory is None:
        print_utility.warn("Service template directory was not provided.  Assuming service-type '{}' is built-in.".format(service_type))
        template = deploy_ctx.template_manager.get_known_service(service_type=service_type)
        if not template:
            print_utility.warn("Service template was not found.  Assuming service-type '{}' is a service modification.".format(service_type))
            template = deploy_ctx.template_manager.get_known_service_modification(service_type=service_type)
        deploy= CloudFormationDeploy(stack_name=deploy_ctx.stack_name, template=template, deploy_ctx=deploy_ctx)
    else:
        deploy= CloudFormationDeploy(stack_name=deploy_ctx.stack_name, template=NamedLocalTemplate(service_template_directory, service_type), deploy_ctx=deploy_ctx)
    errs = deploy.analyze()
    if errs >0:
        print_utility.error("Template raised {} errors ".format(errs),raise_exception=True)




import json
import os

from jsonschema import validate

from infra_buddy.deploy.cloudformation_deploy import CloudFormationDeploy
from infra_buddy.template.template_manager import TemplateManager
from infra_buddy.utility import print_utility

_MODIFICATIONS = 'service-modifications'

_DEPLOYMENT_PARAMETERS = 'deployment-parameters'
_CI_DEPLOYMENT_PARAMETERS = 'ci-deployment-parameters'
_PROD_DEPLOYMENT_PARAMETERS = 'prod-deployment-parameters'
_DEV_DEPLOYMENT_PARAMETERS = 'dev-deployment-parameters'
_UNIT_TEST_DEPLOYMENT_PARAMETERS = 'unit-test-deployment-parameters'
_DOCKER_REGISTRY = 'registry-url'
_SERVICE_TYPE = 'service-type'
_ROLE = 'role'
_APPLICATION = 'application'


class ServiceDefinition(object):
    schema = {
        "type": "object",
        "properties": {
            _APPLICATION: {"type": "string"},
            _ROLE: {"type": "string"},
            _SERVICE_TYPE: {"type": "string"},
            _DOCKER_REGISTRY: {"type": ["string", "null"], 'maxLength': 1000},
            _DEPLOYMENT_PARAMETERS: {
                "type": "object",
                "properties": {
                },
                "additionalProperties": True
            },
            _CI_DEPLOYMENT_PARAMETERS: {
                "type": "object",
                "properties": {
                },
                "additionalProperties": True
            },
            _PROD_DEPLOYMENT_PARAMETERS: {
                "type": "object",
                "properties": {
                },
                "additionalProperties": True
            },
            _DEV_DEPLOYMENT_PARAMETERS: {
                "type": "object",
                "properties": {
                },
                "additionalProperties": True
            }
        },
        "required": [
            _APPLICATION,
            _ROLE,
            _DEPLOYMENT_PARAMETERS,
            _SERVICE_TYPE
        ]
    }

    def __init__(self, artifact_directory, environment):
        super(ServiceDefinition, self).__init__()
        self.artifact_directory = artifact_directory
        service_definition_path = os.path.join(artifact_directory, "service.json")
        if not os.path.exists(service_definition_path):
            err_msg = "Service definition (service.json) does not exist in artifact directory - {}".format(
                artifact_directory)
            print_utility.error(err_msg)
            raise Exception(err_msg)
        with open(service_definition_path, 'r') as fp:
            service_definition = json.load(fp)
            validate(service_definition, self.schema)
            self.application = service_definition[_APPLICATION]
            self.role = service_definition[_ROLE]
            self.service_type = service_definition[_SERVICE_TYPE]
            self.docker_registry = service_definition[_DOCKER_REGISTRY]
            deployment_parameters = _DEPLOYMENT_PARAMETERS
            if deployment_parameters in service_definition:
                self.deployment_parameters = service_definition[deployment_parameters]
            env_deployment_parameters = '{environment}-deployment-parameters'.format(environment=environment)
            if env_deployment_parameters in service_definition:
                self.deployment_parameters.update(service_definition[env_deployment_parameters])
            self.service_modifications = service_definition.get(_MODIFICATIONS, [])

    def generate_execution_plan(self, template_manager, deploy_ctx):
        # type: (TemplateManager) -> list
        ret = []
        ret.append(CloudFormationDeploy(stack_name=deploy_ctx.stack_name,
                                        template=template_manager.get_known_service(self.service_type),
                                        deploy_ctx=deploy_ctx))
        resource_deploy = template_manager.get_resource_service(self.artifact_directory)
        if resource_deploy:
            ret.append(CloudFormationDeploy(stack_name=deploy_ctx.resource_stack_name,
                                            template=resource_deploy,
                                            deploy_ctx=deploy_ctx))
        for mod in self.service_modifications:
            template = template_manager.get_known_service_modification(mod)
            ret.append(CloudFormationDeploy(stack_name=deploy_ctx.generate_modification_stack_name(mod),
                                            template=template,
                                            deploy_ctx=deploy_ctx))
        return ret



import json
import os

from jsonschema import validate

from infra_buddy.deploy.ecs_deploy import ECSDeploy
from infra_buddy.deploy.s3_deploy import S3Deploy
from infra_buddy.utility import print_utility

_ECS_ARTIFACT_TYPE = "ecs"
_S3_ARTIFACT_TYPE = "s3"

_ARTIFACT_TYPE = "artifact-type"
_ARTIFACT_IDENTIFIER = "artifact-identifier"
_ARTIFACT_LOCATION = "artifact-path"



class ArtifactDefinition(object):
    schema = {
        "type": "object",
        "properties": {
            _ARTIFACT_TYPE: {"type": "string", "enum": [_ECS_ARTIFACT_TYPE, _S3_ARTIFACT_TYPE]},
            _ARTIFACT_IDENTIFIER: {"type": "string"},
            _ARTIFACT_LOCATION: {"type": "string"}
        },
        "required": [
            _ARTIFACT_TYPE,
            _ARTIFACT_IDENTIFIER,
            _ARTIFACT_LOCATION
        ]
    }

    def __init__(self, artifact_directory):
        super(ArtifactDefinition, self).__init__()
        definition = self._load_artifact_definition(artifact_directory)
        if not definition:
            definition = self._search_for_legacy_implementation(artifact_directory)
        if definition:
            validate(definition, self.schema)
            self.artifact_type = definition[_ARTIFACT_TYPE]
            self.artifact_location = definition[_ARTIFACT_LOCATION]
            self.artifact_id = definition[_ARTIFACT_IDENTIFIER]
            self.noop = False
        else:
            self.noop = True

    def _search_for_legacy_implementation(self, artifact_directory):
        image_definition = os.path.join(artifact_directory, "containerurl.txt")
        ret = {_ARTIFACT_TYPE: _ECS_ARTIFACT_TYPE}
        if os.path.exists(image_definition):
            print_utility.warn("Defining ECS service udpate with deprecated containerurl.txt artifact")
            with open(image_definition, 'r') as image:
                image_def = self.image = image.read()
                rfind = image_def.rfind(":")
                ret[_ARTIFACT_LOCATION] = image_def[:rfind]
                ret[_ARTIFACT_IDENTIFIER] = image_def[rfind+1:]
            return ret
        return None

    def _load_artifact_definition(self, artifact_directory):
        artifact_def_path = os.path.join(artifact_directory, "artifact.json")
        if os.path.exists(artifact_def_path):
            print_utility.info("Defining artifact definition with artifact.json - {}".format(artifact_def_path))
            with open(artifact_def_path, 'r') as art_def:
                return json.load(art_def)
        else:
            print_utility.warn("Artifact definition (artifact.json) did not exist in artifact directory.")
        return None

    def generate_execution_plan(self, deploy_ctx):
        if self.noop: return None
        if self.artifact_type == _ECS_ARTIFACT_TYPE:
            return ECSDeploy(self.artifact_id, self.artifact_location,deploy_ctx)
        elif self.artifact_type == _S3_ARTIFACT_TYPE:
            return S3Deploy(self.artifact_id,self.artifact_location,deploy_ctx)
        else:
            raise Exception("Well, this is unexpected")
